/*
* PS3000a_JNIWrap
*
* Pico Technology Ltd. 
*
* HSM
*
* September 2011
*/

#include <stdio.h>
#include <jni.h>
#include "ps3000aApi.h"


/*
*
* Function to verify if error has occurred.
*
*/
void checkExceptionOccurred(JNIEnv * jni_env, const char * j_method)
{
	if((*jni_env)->ExceptionOccurred(jni_env)) 
	{
		//printf("Error occurred calling method %s", j_method);
		(*jni_env)->ExceptionDescribe(jni_env);
		(*jni_env)->ExceptionClear(jni_env);
	}
}

//
// Native method to call ps3000aCloseUnit function
//
JNIEXPORT jint JNICALL Java_com_picotech_picoscope_ps3000a_PS3000aJNIWrapperAccess_ps3000aCloseUnit
  (JNIEnv * env, jobject jobj, jshort js_handle)
{
	return ps3000aCloseUnit(js_handle);
}


//
// Native method to call ps3000aOpenUnit function
//
JNIEXPORT jint JNICALL Java_com_picotech_picoscope_ps3000a_PS3000aJNIWrapperAccess_ps3000aOpenUnit
  (JNIEnv * env, jobject jobj, jobject jobj_ou_params)
{
	//Function variables
	jclass jc_open_unit_params;

	jmethodID jmid_set_handle;
	
	jint pico_status;
	jshort js_handle;


	//Obtain class instance
	jc_open_unit_params = (*env)->GetObjectClass(env, jobj_ou_params);

	//Obtain method IDs
	jmid_set_handle = (*env)->GetMethodID(env, jc_open_unit_params, "setHandle", "(S)V");
	

	//Check if method IDs returned
	
	if(jmid_set_handle == 0)
	{
		printf("Can't find method setHandle.\n");
		return 0;
	}

	//Open the unit
	pico_status = ps3000aOpenUnit(&js_handle, NULL);


	//Set the values
	(*env)->ExceptionClear(env);

	(*env)->CallVoidMethod(env, jobj_ou_params, jmid_set_handle, js_handle);
	checkExceptionOccurred(env, "setHandle");

	return pico_status;
}

//
// Native method to call ps3000aGetUnitInfo function
//
JNIEXPORT jint JNICALL Java_com_picotech_picoscope_ps3000a_PS3000aJNIWrapperAccess_ps3000aGetUnitInfo
  (JNIEnv * env, jobject jobj, jshort js_handle, jobject jobj_unit_info)
{

	//Class instance
	jclass jc_unit_info;

	//Method IDs
	jmethodID jmid_set_driver_version;
	jmethodID jmid_set_usb_version;
	jmethodID jmid_set_hardware_version;
	jmethodID jmid_set_variant_info;
	jmethodID jmid_set_batch_and_serial;
	jmethodID jmid_set_calibration_date;
	jmethodID jmid_set_kernel_version;
	jmethodID jmid_set_digital_hw_version;
	jmethodID jmid_set_analogue_hw_version;

	jint ji_pico_status;
	jstring jstr_info_string;

	char info_line[80];

	int i;
	short required_length = 0;

	//printf("DEBUG: Obtaining class instance...");

	//Get class instance
	jc_unit_info = (*env)->GetObjectClass(env, jobj_unit_info);

	//printf("DEBUG: Obtaining method IDs...");

	//Get method IDs
	jmid_set_driver_version = (*env)->GetMethodID(env, jc_unit_info, "setDriverVersion", "(Ljava/lang/String;)V");
	jmid_set_usb_version = (*env)->GetMethodID(env, jc_unit_info, "setUsbVersion", "(Ljava/lang/String;)V");
	jmid_set_hardware_version = (*env)->GetMethodID(env, jc_unit_info, "setHardwareVersion", "(Ljava/lang/String;)V");
	jmid_set_variant_info = (*env)->GetMethodID(env, jc_unit_info, "setVariantInfo", "(Ljava/lang/String;)V");
	jmid_set_batch_and_serial = (*env)->GetMethodID(env, jc_unit_info, "setBatchAndSerial", "(Ljava/lang/String;)V");
	jmid_set_calibration_date = (*env)->GetMethodID(env, jc_unit_info, "setCalibrationDate", "(Ljava/lang/String;)V");
	jmid_set_kernel_version = (*env)->GetMethodID(env, jc_unit_info, "setKernelVersion", "(Ljava/lang/String;)V");
	jmid_set_digital_hw_version = (*env)->GetMethodID(env, jc_unit_info, "setDigitalHardwareVersion", "(Ljava/lang/String;)V");
	jmid_set_analogue_hw_version = (*env)->GetMethodID(env, jc_unit_info, "setAnalogueHardwareVersion", "(Ljava/lang/String;)V");

	printf("DEBUG: Checking method IDs...");

	//Check method IDs found
	if(jmid_set_driver_version == 0)
	{
		printf("Can't find method setDriverVersion.\n");
		return 0;
	}

	if(jmid_set_usb_version == 0)
	{
		printf("Can't find method setUsbVersion.\n");
		return 0;
	}

	if(jmid_set_hardware_version == 0)
	{
		printf("Can't find method setHardwareVersion.\n");
		return 0;
	}

	if(jmid_set_variant_info == 0)
	{
		printf("Can't find method setVariantInfo.\n");
		return 0;
	}

	if(jmid_set_batch_and_serial == 0)
	{
		printf("Can't find method setBatchAndSerial.\n");
		return 0;
	}

	if(jmid_set_calibration_date == 0)
	{
		printf("Can't find method setCalibrationDate.\n");
		return 0;
	}

	if(jmid_set_kernel_version == 0)
	{
		printf("Can't find method setKernelVersion.\n");
		return 0;
	}

	if(jmid_set_digital_hw_version == 0)
	{
		printf("Can't find method setDigitalHardwareVersion.\n");
		return 0;
	}

	if(jmid_set_analogue_hw_version == 0)
	{
		printf("Can't find method setAnalogueHardwareVersion.\n");
		return 0;
	}

	//printf("DEBUG: Obtaining unit information...");

	//Call methods
	for(i = 0; i < PICO_ANALOGUE_HARDWARE_VERSION + 1; i = i + 1)
	{
		ji_pico_status = ps3000aGetUnitInfo(js_handle, info_line, sizeof(info_line), &required_length, i);

		jstr_info_string = (*env)->NewStringUTF(env, info_line);

		printf("i %d, pico status %d", i, ji_pico_status);

		(*env)->ExceptionClear(env);

		switch(i)
		{
			case PICO_DRIVER_VERSION:
			{
				(*env)->CallVoidMethod(env, jobj_unit_info, jmid_set_driver_version, jstr_info_string);
				checkExceptionOccurred(env, "setDriverVersion");
				break;
			}
			case PICO_USB_VERSION:
			{
				(*env)->CallVoidMethod(env, jobj_unit_info, jmid_set_usb_version, jstr_info_string);
				checkExceptionOccurred(env, "setUSBVersion");
				break;
			}
			case PICO_HARDWARE_VERSION:
			{
				(*env)->CallVoidMethod(env, jobj_unit_info, jmid_set_hardware_version, jstr_info_string);
				checkExceptionOccurred(env, "setHardwareVersion");
				break;
			}
			case PICO_VARIANT_INFO:
			{
				(*env)->CallVoidMethod(env, jobj_unit_info, jmid_set_variant_info, jstr_info_string);
				checkExceptionOccurred(env, "setVariantInfo");
				break;
			}
			case PICO_BATCH_AND_SERIAL:
			{
				(*env)->CallVoidMethod(env, jobj_unit_info, jmid_set_batch_and_serial, jstr_info_string);
				checkExceptionOccurred(env, "setBatchAndSerial");
				break;
			}
			case PICO_CAL_DATE:
			{
				(*env)->CallVoidMethod(env, jobj_unit_info, jmid_set_calibration_date, jstr_info_string);
				checkExceptionOccurred(env, "setCalibrationDate");
				break;
			}
			case PICO_KERNEL_VERSION:
			{
				(*env)->CallVoidMethod(env, jobj_unit_info, jmid_set_kernel_version, jstr_info_string);
				checkExceptionOccurred(env, "setKernelVersion");
				break;
			}
			case PICO_DIGITAL_HARDWARE_VERSION:
			{
				(*env)->CallVoidMethod(env, jobj_unit_info, jmid_set_digital_hw_version, jstr_info_string);
				checkExceptionOccurred(env, "setDigitalHardwareVersion");
				break;
			}
			case PICO_ANALOGUE_HARDWARE_VERSION:
			{
				(*env)->CallVoidMethod(env, jobj_unit_info, jmid_set_analogue_hw_version, jstr_info_string);
				checkExceptionOccurred(env, "setAnalogueHardwareVersion");
				break;
			}
			default:
			{
				printf("Invalid information request");
			}

		}

		//(*env)->ReleaseStringUTFChars(env, jstr_info_string, info_line);

	}

	return ji_pico_status;
}


